import { Sql, TransactionSql } from "postgres";


type MigrationFunc = (sql:TransactionSql)=>Promise<void>

interface Migration {
  name: string
  up: MigrationFunc
  // TODO: implement down
  down?: MigrationFunc
}

const migration = (name:string, up:MigrationFunc, down?:MigrationFunc)=>{
  return {name, up, down}
}

const migrations: Array<Migration> = [
  migration("create-api-responses", async (sql)=>{
    await sql`
create table if not exists wynn_api_responses (
  path text not null,
  content jsonb not null,
  content_hash text not null,
  received_time timestamptz not null
)`
    await sql`
create index wynn_api_responses_received_time on wynn_api_responses (received_time, path)
`
  }),
  migration("guild-info", async (sql)=>{
    await sql`
create table if not exists wynn_guild_members (
  guild_id UUID not null,
  member_id UUID not null,
  rank text not null,
  joined_at timestamptz not null,
  contributed bigint not null,
  primary key (guild_id, member_id),
  unique(member_id)
)`
    await sql`
create table if not exists wynn_guild_info (
  uid UUID not null,
  name text not null,
  prefix text not null,
  level bigint ,
  xp_percent bigint,
  territories bigint,
  wars bigint,
  created timestamptz,
  primary key (uid)
)`

    await sql`
create table if not exists wynn_guild_season_results (
  guild_id UUID not null,
  season text not null,
  rating bigint not null,
  territories bigint not null,
  primary key (guild_id, season)
)
`
  }),
  migration("create-user-info", async (sql)=>{
    await sql`
  create table if not exists minecraft_user (
    uid UUID not null,
    name text not null,
    server text,
    primary key (uid)
  )
  `
  }),
  migration("create-guild-settings", async (sql)=>{
    await sql`
  create table if not exists discord_guild_setting(
    discord_guild text not null,
    name text not null,
    value jsonb not null,
    primary key (discord_guild, name)
  )`
  })

]


export const runMigrations = async (pg: Sql) => {
  await pg.begin(async (sql)=>{
    await sql`
create table if not exists migration_version (version int)
`
    await sql`
insert into migration_version
  select 0
    where 0=(select count(*) from migration_version)
`
    const [{version}] =  await sql`
select version from migration_version limit 1
`

    const targetVersion = migrations.length

    if(version == targetVersion) {
      return
    }
    if (version > targetVersion) {
      console.log(`version ${version} is greater than the latest migration ${targetVersion}, nothing to do`)
      return
    }
    console.log(`running migrations from version ${version}`)
    for (let i = version+1; i <= targetVersion; i++) {
      const m = migrations[i-1]
      console.log(`running migration ${i}_${m.name}`)
      await m.up(sql)
      await sql`
update migration_version set version=${i}
`
    }
    console.log(`done running db migrations`)
  });
}
